=== modified file 'NEWS'
--- NEWS	2011-09-08 00:33:28 +0000
+++ NEWS	2011-11-02 01:30:45 +0000
@@ -42,6 +42,9 @@
     - Fix support for displaying foreign revision ids.
       (Jelmer Vernooij, #736026)
 
+    - Add hook 'controller' to BranchWSGIApp, allowing other bzr plugins
+      to provide custom branch-specific controllers. (Jelmer Vernooij, #585822)
+
 
 1.18.1 [24Mar2011]
 ------------------

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2011-10-31 20:48:56 +0000
+++ loggerhead/apps/branch.py	2011-11-04 18:16:45 +0000
@@ -22,6 +22,7 @@
 
 import bzrlib.branch
 import bzrlib.errors
+from bzrlib.hooks import Hooks
 import bzrlib.lru_cache
 
 from paste import request
@@ -162,6 +163,10 @@
                     self.served_url = self.url([])
                 except bzrlib.errors.InvalidURL:
                     self.served_url = None
+        for hook in self.hooks['controller']:
+            controller = hook(self, environ)
+            if controller is not None:
+                return controller
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
@@ -172,9 +177,9 @@
             environ['loggerhead.as_json'] = True
             path = request.path_info_pop(environ)
         cls = self.controllers_dict.get(path)
-        if cls is None:
-            raise httpexceptions.HTTPNotFound()
-        return cls(self, self.get_history)
+        if cls is not None:
+            return cls(self, self.get_history)
+        raise httpexceptions.HTTPNotFound()
 
     def app(self, environ, start_response):
         self.branch.lock_read()
@@ -188,3 +193,23 @@
                 raise
         finally:
             self.branch.unlock()
+
+
+class BranchWSGIAppHooks(Hooks):
+    """A dictionary mapping hook name to a list of callables for WSGI app branch hooks.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+        """
+        Hooks.__init__(self, "bzrlib.plugins.loggerhead.apps.branch",
+            "BranchWSGIApp.hooks")
+        self.add_hook('controller',
+            "Invoked when looking for the controller to use for a "
+            "branch subpage. The api signature is (branch_app, environ)."
+            "If a hook can provide a controller, it should return one, "
+            "as a standard WSGI app. If it can't provide a controller, "
+            "it should return None", (1, 19))
+
+
+BranchWSGIApp.hooks = BranchWSGIAppHooks()

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2011-09-02 09:43:08 +0000
+++ loggerhead/tests/test_controllers.py	2011-11-02 01:56:02 +0000
@@ -1,11 +1,7 @@
-import simplejson
-
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
-from loggerhead.controllers.revision_ui import RevisionUI
 from loggerhead.tests.test_simple import BasicTests, consume_app
-from loggerhead import util
 
 
 class TestInventoryUI(BasicTests):
@@ -168,6 +164,7 @@
         annotated = annotate_info['annotated']
         self.assertEqual(0, len(annotated))
 
+
 class TestFileDiffUI(BasicTests):
 
     def make_branch_app_for_filediff_ui(self):
@@ -233,3 +230,31 @@
         revlog_ui = branch_app.lookup_app(env)
         self.assertOkJsonResponse(revlog_ui, env)
 
+
+class TestControllerHooks(BasicTests):
+
+    def test_dummy_hook(self):
+        return
+        # A hook that returns None doesn't influence the searching for
+        # a controller.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: None
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertRaises(KeyError, app.lookup_app, env)
+
+    def test_working_hook(self):
+        # A hook can provide an app to use for a particular request.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: "I am hooked"
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertEquals("I am hooked", app.lookup_app(env))

